package org.openliberty.openaz.pep;

import java.util.Date;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openliberty.openaz.azapi.AzAttribute;
import org.openliberty.openaz.azapi.AzAttributeValueString;
import org.openliberty.openaz.azapi.AzEntity;
import org.openliberty.openaz.azapi.constants.AzCategoryId;
import org.openliberty.openaz.azapi.constants.AzCategoryIdAction;
import org.openliberty.openaz.azapi.constants.AzCategoryIdEnvironment;
import org.openliberty.openaz.azapi.constants.AzCategoryIdResource;
import org.openliberty.openaz.azapi.constants.AzCategoryIdSubjectAccess;
import org.openliberty.openaz.azapi.constants.AzDataTypeIdString;
import org.openliberty.openaz.azapi.constants.AzXacmlStrings;
/**
 * Default implementation of <code>JavObjectMapper</code>.  
 * It supports the mapping of 
 * <ul>
 * <li>
 * <code>java.util.Date</code>,
 * <li>
 * <code>java.lang.String</code>, and 
 * <li>
 * <code>java.util.Map</code>
 * </ul>
 * to <code>AzWrapperRequestObject</code>
 * @author Josh Bregman, Rich Levinson, Prateek Mishra
 */
public class SimpleJavaObjectMapper implements JavaObjectMapper {
	/**
	 * Set of class objects supported by this mapper
	 */
	private static Set<Class> supportedClasses =
		new HashSet<Class>() {{ add(Date.class); 
								add(String.class); 
								add(Map.class); }};
	Log log = LogFactory.getLog(this.getClass()); 
	
	/**
	 * Create a SimpleJavaObjectMapper
	 */
    public SimpleJavaObjectMapper() {
        super();
    }

    /**
     * Returns the set of class objects supported by this Mapper
     */
    public Set<Class> getSupportedClasses(){
    	return supportedClasses;
    }
    
    /**
     * Determines if this Mapper can map the type of class that
     * that is passed in as a parameter. Basically involves checking
     * that the class matches one of a set of known classes that
     * can be mapped.
     */
    public boolean canMapObject(Object obj) {
    	log.info("canMap(Object obj): " + obj.getClass().getName());
        if (obj instanceof Date){
        	log.info("obj instanceof Date == true");
            return true;
        } else if (obj instanceof String) {
        	log.info("obj instanceof String == true");
       	return true;
        } else if (obj instanceof Map) {
        	log.info("obj instanceof Map == true");
        	return true;
        } else if (obj instanceof HashMap) {
        	log.info("obj instanceof HashMap == true");
        	return true;
        } else if (obj instanceof String) {
        	return true;
        } else {
        	log.info("Cannot map obj instanceof: " + obj.getClass().getName());
            return false;
        }
    }

    /**
     * Maps a Java Object to one or more AzAttributes. In general
     * a Java Object can contain several attributes that may be
     * needed as AzAttributes in the decision process.
     */
    public <T extends Enum<T> & AzCategoryId> 
    	AzWrapperRequestObject<T> map(
    			Object javaObject,
                AzWrapperRequestObject<T> azWrapperObject) 
            throws PepException{
        log.info("Object to map: " + 
        			javaObject.getClass().getName());
        if (javaObject instanceof Date) {
            return convertDate((Date)javaObject,azWrapperObject);
        } else if (javaObject instanceof String) {
            return convertString(javaObject,azWrapperObject);
        } else if (javaObject instanceof Map) {
            return convertMap(javaObject,azWrapperObject);
        }
        
        throw new PepException("Can't map an object of class "+javaObject.getClass().getName());
        
    }

    /**
     * Converts a Date to an environment AzEntity date attribute
     * @param <T>
     * @param date
     * @param azWrapperObject
     * @return
     */
    private <T extends Enum<T> & AzCategoryId> 
    		AzWrapperRequestObject<T> convertDate(
    				Date date,
                    AzWrapperRequestObject<T> azWrapperObject) {
        //if (azWrapperObject instanceof Environment) {
        //    Environment env = (Environment)azWrapperObject;
        //    env.setCurrentDateTime(date);
        if (azWrapperObject.wrappedObject.getAzCategoryId().equals(
        		AzCategoryIdEnvironment.AZ_CATEGORY_ID_ENVIRONMENT)) {
        	String attributeId = AzXacmlStrings.X_ATTR_ENV_CURRENT_DATE_TIME;
            azWrapperObject.setAttribute(attributeId, date);
        }
        
        return azWrapperObject;
    }

    /**
     * Converts a Java object to a String value and names it
     * with an AttributeId associated with the Id attribute
     * conventially used with the associated AzEntity type.
     * @param <T>
     * @param javaObject
     * @param azWrapperObject
     * @return
     * @throws PepException
     */
    private <T extends Enum<T> & AzCategoryId> 
    	AzWrapperRequestObject<T> convertString(
    			Object javaObject,
                AzWrapperRequestObject<T> azWrapperObject) 
            throws PepException {
        
        String attributeId = null;
        
        //if (azWrapperObject instanceof Subject) {
        if (azWrapperObject.wrappedObject.getAzCategoryId().equals(
        		AzCategoryIdSubjectAccess.AZ_CATEGORY_ID_SUBJECT_ACCESS)) {
            attributeId = AzXacmlStrings.X_ATTR_SUBJECT_ID;
        } else if (azWrapperObject.wrappedObject.getAzCategoryId().equals(
        		AzCategoryIdResource.AZ_CATEGORY_ID_RESOURCE)) {
            attributeId = AzXacmlStrings.X_ATTR_RESOURCE_ID;
        } else if (azWrapperObject.wrappedObject.getAzCategoryId().equals(
        		AzCategoryIdAction.AZ_CATEGORY_ID_ACTION)) {
            attributeId = AzXacmlStrings.X_ATTR_ACTION_ID;
        } else {
            throw new PepException("Can't Convert String for "+azWrapperObject.getWrappedObject().getAzCategoryId().name());
        }
        
        azWrapperObject.setAttribute(attributeId,(String)javaObject);
        
        return azWrapperObject;
        
       
    }

    /**
     * Converts a Map of String pairs to one AzAttribute Id and value
     * associated with each of the String pairs.
     * @param <T>
     * @param javaObject
     * @param azWrapperRequestObject
     * @return
     */
    private <T extends Enum<T> & AzCategoryId> 
    	AzWrapperRequestObject<T> convertMap(
						Object javaObject,
                        AzWrapperRequestObject<T> azWrapperRequestObject) {
        
        Map<String,String> map = (Map<String,String>)javaObject;
        
        Iterator<Map.Entry<String,String>> it = 
        	map.entrySet().iterator();
        
        while (it.hasNext()) {
            Map.Entry<String,String> entry = it.next();
            azWrapperRequestObject.setAttribute(
            		entry.getKey(),entry.getValue());
        }
        
        return azWrapperRequestObject;
        
    }
}
