package test;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Iterator;
import java.util.Date;
import java.util.Map;
import java.util.Properties;
import java.io.FilePermission;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.security.DomainCombiner;
//import javax.security.auth.Subject;
import java.security.Principal;
import javax.security.auth.x500.X500Principal;
import javax.security.auth.x500.X500PrivateCredential;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openliberty.openaz.azapi.AzService;
import org.openliberty.openaz.azapi.AzRequestContext;
import org.openliberty.openaz.azapi.AzResourceActionAssociation;
import org.openliberty.openaz.azapi.AzResourceActionAssociationId;
import org.openliberty.openaz.azapi.AzAttribute;
import org.openliberty.openaz.azapi.constants.AzXacmlStrings;
import org.openliberty.openaz.pep.Action;
import org.openliberty.openaz.pep.Obligation;
//import org.openliberty.openaz.pep.ObligationIterator;
//import org.openliberty.openaz.pep.ObligationNameValuePair;
import org.openliberty.openaz.pep.JavaObjectMapper;
import org.openliberty.openaz.pep.PepRequestFactory;
import org.openliberty.openaz.pep.PepRequest;
import org.openliberty.openaz.pep.PepResponse;
import org.openliberty.openaz.pep.PepException;
import org.openliberty.openaz.pep.Resource;
import org.openliberty.openaz.pep.Subject;
import org.openliberty.openaz.pep.SimpleJavaPermissionMapper;
import org.openliberty.openaz.pep.SimpleJavaObjectMapper;
import test.objects.TestRolePrincipal;
import test.objects.TestJaasSubjectMapper;
import test.TestUtils;

public class TestStyles {
	
	// defn: "container": manages "environment" and "issues" env attributes
	public final static String CONTAINER = "orcl-weblogic";   // [a03]
	
	// defn: "application": manages "resources" and "issues" resource attrs
	public final static String APPLICATION = "hr-application-01"; // [a04]
	
	// defn: az provider class name
	public final static String PROVIDER_CLASS_NAME = 
		"org.openliberty.openaz.pdp.provider.SimpleConcreteService";
	
	// test user; normally would be obtained from session info
	public final static String SAMPLE_SESSION_USER_NAME = "Joe User";
	public final static String SAMPLE_SESSION_AUTH_METHOD = "password";
	
	// defns for attr methods:
	public final static String SUBJ_AUTH_METH_ATTR_ID = 
		AzXacmlStrings.X_ATTR_SUBJECT_AUTHN_LOC_AUTHENTICATION_METHOD;
	public final static String PWD_AUTH_METH = "password";
	public final static String RES_TYPE_ATTR_ID = "resource-type-attr-id";
	public final static String FILE_RES_TYPE = "file";
	Log log = LogFactory.getLog(this.getClass()); 
	static TestUtils testUtils = new TestUtils();
	/** Main test method
	 * 
	 */
	public static void main(String[] args) throws Exception {
		TestStyles ts = new TestStyles();
		ts.testStyleString();
		ts.testStyleMappers();
		ts.testStyleSubjectMapper();
	}

	/**
	 * String style 1
	 *  i. simple strings, process obligations
	 *  
	 * @throws Exception
	 */
	public void testStyleString() throws Exception{
		printTestHeader();
		System.out.println(
				"\n&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n" +
				"      Running Test Style: String          \n" +
				"&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n");
		log.info("Begin test style: String");
        AzService az = new org.openliberty.openaz.pdp.provider.SimpleConcreteService();
		if (!(az==null)){
			PepRequestFactory pepReqFactory = 
				new PepRequestFactory(CONTAINER,az);                
 			String subjectId = SAMPLE_SESSION_USER_NAME;
 			String resourceId = "file:C\\\\toplevel";
 			String actionId = "read";
			PepRequest pepReq = 
				pepReqFactory.newPepRequest(
						subjectId, resourceId, actionId); 
			log.info("pepReq created w subject,resource,action: " +
					subjectId + ", " + resourceId + ", " + actionId);
			PepResponse pepRspCtx = pepReq.decide();
			
			if (pepRspCtx != null) {
				log.info("TestStyles: Strings; allowed() = " +
							pepRspCtx.allowed());
				testUtils.logObligations(pepRspCtx);
			}
			else
				log.info("pepRspCtx is unexpectedly null - need to investigate");
		}
		else
			log.info("az == null");		
	}
	

	/**
	 * 		String + attr style 1a
	 * ii. convert from JAAS Subject, Permission
	 * 
	 * Set the mappers that will be used by each category:
	 * Subject, Action, Resource, Environment.
	 * The list of mappers for each category defines the objects
	 * that may be submitted for that category and how the
	 * information in those objects get mapped to xacml
	 * AttributeId and AttributeValue.
	 * 
	 * @throws Exception
	 */
	public void testStyleMappers() throws Exception {
		printTestHeader();
		System.out.println(
		"\n&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n" +
		"      Running Test Style: Mappers + Attributes          \n" +
		"&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n");
		log.info("Begin test style: Mappers");
        AzService az = new org.openliberty.openaz.pdp.provider.SimpleConcreteService();
		if (!(az==null)){
			PepRequestFactory pepReqFactory = 
				new PepRequestFactory(CONTAINER,az);
			List<JavaObjectMapper> objectMapper = 
		    	Arrays.asList(new JavaObjectMapper[] {
		    			new SimpleJavaObjectMapper(),
		    			new SimpleJavaPermissionMapper()});
			List<JavaObjectMapper> permissionMapper = 
		    	Arrays.asList(new JavaObjectMapper[] {
		    			new SimpleJavaPermissionMapper()});
			pepReqFactory.getActionFactory().setMappers(permissionMapper);
			pepReqFactory.getResourceFactory().setMappers(permissionMapper);
			pepReqFactory.getSubjectFactory().setMappers(objectMapper);
			pepReqFactory.getEnvironmentFactory().setMappers(objectMapper);
			
			log.info("Subject Mapper supported classes: " +
					pepReqFactory.getSubjectFactory().getMappers().
						iterator().next().getSupportedClasses());
			log.info("Subject Mapper supported classes: " +
					pepReqFactory.getSubjectFactory().getSupportedClasses());
						
            HashMap<String,String> subject = new HashMap<String,String>();           
            subject.put(AzXacmlStrings.X_ATTR_SUBJECT_ID,"josh");
            subject.put(AzXacmlStrings.X_ATTR_SUBJECT_AUTHN_LOC_AUTHENTICATION_METHOD,SAMPLE_SESSION_AUTH_METHOD);
	 			
			String file = "file:///C/toplevel/permissionTest";
			String action = "Read";
			FilePermission filePermission = new FilePermission(file, action);
                        
			Date now = new Date();
			
			PepRequest pepReq = 
				pepReqFactory.newPepRequest(
						subject, filePermission, now); 
			log.info("pepReq created w : " +
					"Subject as Object Hashmap, "  + 
					"Resource/Action as Object FilePermission, and " + 
					"Environment as Object: Date");
			PepResponse pepRspCtx = pepReq.decide();
			
			if (pepRspCtx != null) {
				log.info("TestStyles: Mappers; allowed() = " +
							pepRspCtx.allowed());
				testUtils.logObligations(pepRspCtx);
			}
			else
				log.info("pepRsp seems to be null");
		}
		else
			log.info("az == null");		
	}    
	
	/**
	 * JAAS Subject style 
	 * 
	 * iii. Adding additional context information from
	 * the JAAS Subject
	 * 
	 * @throws Exception
	 */
	public  void testStyleSubjectMapper() throws Exception {
		printTestHeader();
		System.out.println(
				"\n&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n" +
				"      Running Test Style: JAAS Subject        \n" +
				"&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n");
		log.info("Begin test style: Mappers");

		// Create some dummy Principals
	    Set<Principal> xPrins = new HashSet<Principal>();
	    xPrins.add(new X500Principal("CN=Rich, OU=Identity Management, O=Oracle, C=US"));
	    xPrins.add(new TestRolePrincipal("developer"));

	    // Create some dummy Credential containers
	    Set<X500PrivateCredential> xCreds = new HashSet<X500PrivateCredential>();
	    Set<X500PrivateCredential> xPubCreds = new HashSet<X500PrivateCredential>();
	    
	    // Create a dummy JAAS Subject
	    javax.security.auth.Subject jaasSubject = 
	    	new javax.security.auth.Subject(false, xPrins, xCreds, xPubCreds);
	    
	    // Get Principals from the JAAS Subject
	    Set<? extends Principal> yPrins = jaasSubject.getPrincipals();
	    Iterator<? extends Principal> itPrin = yPrins.iterator();
	    
	    AzService az = new org.openliberty.openaz.pdp.provider.SimpleConcreteService();
		if (!(az==null)){
			PepRequestFactory pepReqFactory = 
				new PepRequestFactory(CONTAINER,az);
			// Create 3 Mappers using different Mapper Impls
			List<JavaObjectMapper> subjectMapper = 
		    	Arrays.asList(new JavaObjectMapper[] {
		    			new TestJaasSubjectMapper()});
			List<JavaObjectMapper> objectMapper = 
		    	Arrays.asList(new JavaObjectMapper[] {
		    			new SimpleJavaObjectMapper()});
			List<JavaObjectMapper> permissionMapper = 
		    	Arrays.asList(new JavaObjectMapper[] {
		    			new SimpleJavaPermissionMapper()});
			// Set the appropriate Mappers on respective factories
			pepReqFactory.getActionFactory().setMappers(permissionMapper);
			pepReqFactory.getResourceFactory().setMappers(permissionMapper);
			pepReqFactory.getSubjectFactory().setMappers(subjectMapper);
			pepReqFactory.getEnvironmentFactory().setMappers(objectMapper);
			// Create a FilePermission as a resource to request
			String file = "file:///C/toplevel/permissionTest";
			String action = "Read";
			FilePermission filePermission = new FilePermission(file, action);
              
			// An environment attribute
			Date now = new Date();
			
			// Create a single decision 3 param request
			PepRequest pepReq = 
				pepReqFactory.newPepRequest(
						jaasSubject, filePermission, now); 
			log.info("pepReq created w : " +
					"Subject as Object Hashmap, "  + 
					"Resource/Action as Object FilePermission, and " + 
					"Environment as Object: Date");
			PepResponse pepRspCtx = pepReq.decide();
			
			if (pepRspCtx != null) {
				log.info("TestStyles: Mappers; allowed() = " +
							pepRspCtx.allowed());
				testUtils.logObligations(pepRspCtx);
			}
			else
				log.info("pepRsp seems to be null");
		}
	}
	public void printTestHeader() {
		System.out.println("\n\n" +
			"\n***********************************************************************" +
			"\n***********************************************************************" +
			"\n***********************************************************************" +
			"\n\n");
	}
}
